<?php

namespace Drupal\Tests\cloud_budget\Functional;

/**
 * Tests cloud credit.
 *
 * @group Cloud
 */
class CloudCreditTest extends CloudBudgetTestCase {

  public const CLOUD_CREDIT_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list cloud credit',
      'view cloud credit',
      'edit cloud credit',
      'add cloud credit',
      'delete cloud credit',
    ];
  }

  /**
   * Tests CRUD for Cloud Credit.
   */
  public function testCloudCredit() {

    $cloud_context = $this->cloudContext;

    // List Cloud credit for Cloud Budget.
    $this->drupalGet("/clouds/budget/$cloud_context/credit");
    $this->assertSession()->statusCodeEquals(200);

    // Add a new Cloud Credit.
    $add = $this->createCloudCreditTestFormData(self::CLOUD_CREDIT_REPEAT_COUNT);
    for ($i = 0; $i < self::CLOUD_CREDIT_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $name = explode(' ', $add[$i]['user[0][target_id]'])[0];
      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/budget/$cloud_context/credit/add",
        $add[$i],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('The Cloud Credit "@name" has been created.', ['@name' => $name]));

      // Make sure listing.
      $this->drupalGet("/clouds/budget/$cloud_context/credit");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($name);
    }

    // Edit a Cloud Credit.
    $edit = $this->createCloudCreditTestFormData(self::CLOUD_CREDIT_REPEAT_COUNT);
    for ($i = 0; $i < self::CLOUD_CREDIT_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      unset($edit[$i]['user[0][target_id]']);
      $this->drupalPostForm(
        "/clouds/budget/$cloud_context/credit/$num/edit",
        $edit[$i],
        $this->t('Save')
      );

      $name = explode(' ', $add[$i]['user[0][target_id]'])[0];

      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Cloud Credit "@name" has been saved.',
        ['@name' => $name]
      ));
    }

    // Delete Cloud Credit.
    for ($i = 0; $i < self::CLOUD_CREDIT_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $name = explode(' ', $add[$i]['user[0][target_id]'])[0];

      $this->drupalPostForm(
        "/clouds/budget/$cloud_context/credit/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Cloud Credit "@name" has been deleted.',
        ['@name' => $name]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/budget/$cloud_context/credit");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($name);
    }
  }

  /**
   * Tests deleting cloud credits with bulk operation.
   */
  public function testCloudCreditBulk() {
    for ($i = 0; $i < self::CLOUD_CREDIT_REPEAT_COUNT; $i++) {
      // Create Cloud Credits.
      $cloud_credits = $this->createCloudCreditsRandomTestFormData();
      $entities = [];
      foreach ($cloud_credits ?: [] as $cloud_credit) {
        $entities[] = $this->createCloudCreditTestEntity($cloud_credit);
      }

      $this->doTestEntityBulk('credit', $entities);
    }
  }

}
