<?php

namespace Drupal\Tests\cloud_budget\Traits;

use Drupal\cloud_budget\Entity\CloudCredit;

/**
 * The trait creating test entity for cloud budget testing.
 */
trait CloudBudgetTestEntityTrait {

  /**
   * Test bulk operation for entities.
   *
   * @param string $entity_type_name
   *   The name of the entity type. For example, instance.
   * @param array $entities_data
   *   The data of entities.
   * @param string $operation
   *   The operation.
   * @param string $operation_passive
   *   The passive voice of operation.
   * @param string $path_prefix
   *   The URL path of prefix.
   */
  protected function doTestEntityBulk($entity_type_name,
                                      array $entities_data,
                                      $operation = 'delete',
                                      $operation_passive = 'deleted',
                                      $path_prefix = '/clouds/budget') {

    $cloud_context = !empty($this->cloudContext)
      ? '/' . $this->cloudContext
      : '';

    $entity_count = count($entities_data);

    $entity_type_id = $entities_data[0]->getEntityTypeId();
    $data = [];
    $data['action'] = "${entity_type_id}_${operation}_action";

    $this->drupalGet("$path_prefix$cloud_context/$entity_type_name");

    $checkboxes = $this->cssSelect('input[type=checkbox]');
    foreach ($checkboxes ?: [] as $checkbox) {
      if ($checkbox->getAttribute('name') === NULL) {
        continue;
      }

      $data[$checkbox->getAttribute('name')] = $checkbox->getAttribute('value');
    }

    // Confirm.
    $this->drupalPostForm(
      "$path_prefix$cloud_context/$entity_type_name",
      $data,
      $this->t('Apply to selected items')
    );
    $this->assertSession()->statusCodeEquals(200);

    $message = \Drupal::translation()->formatPlural($entity_count,
      'Are you sure you want to @operation this @item?',
      'Are you sure you want to @operation these @item entities?', [
        '@operation' => $operation,
        '@item' => $entities_data[0]->getEntityType()->getSingularLabel(),
      ]
    );

    $this->assertSession()->pageTextContains($message);
    foreach ($entities_data ?: [] as $entity_data) {
      $entity_name = $entity_data->label();
      $this->assertSession()->pageTextContains($entity_name);
    }

    // Operation.
    $operation_upper = ucfirst($operation);
    $this->drupalPostForm(
      "$path_prefix$cloud_context/$entity_type_name/${operation}_multiple",
      [],
      $operation_upper
    );

    $this->assertSession()->statusCodeEquals(200);

    foreach ($entities_data ?: [] as $entity_data) {
      $this->assertSession()->pageTextContains($this->t('The @type "@label" has been @operation_passive.', [
        '@type' => $entity_data->getEntityType()->getLabel(),
        '@label' => $entity_data->label(),
        '@operation_passive' => $operation_passive,
      ]));
    }

    $operation_passive_upper = ucfirst($operation_passive);
    $message = \Drupal::translation()->formatPlural($entity_count,
      $this->t('@operation_passive_upper @entity_count item.', [
        '@operation_passive_upper' => $operation_passive_upper,
        '@entity_count' => $entity_count,
      ]),
      $this->t('@operation_passive_upper @entity_count items.', [
        '@operation_passive_upper' => $operation_passive_upper,
        '@entity_count' => $entity_count,
      ])
    );
    $this->assertSession()->pageTextContains($message);
  }

  /**
   * Create a Cloud Credit test entity.
   *
   * @param array $cloud_credit
   *   The cloud credit data.
   *
   * @return \Drupal\cloud_budget\Entity\CloudCredit
   *   The cloud credit entity.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function createCloudCreditTestEntity(array $cloud_credit) {
    $entity = CloudCredit::create([
      'cloud_context' => $cloud_credit['cloud_context'],
      'user' => $cloud_credit['user'],
      'amount' => $cloud_credit['amount'],
      'refreshed' => time(),
    ]);
    $entity->save();
    return $entity;
  }

}
