<?php

namespace Drupal\Tests\aws_cloud\Functional\Vpc;

use Drupal\Tests\aws_cloud\Functional\AwsCloudTestCase;

/**
 * Tests AWS Cloud VPC Peering Connection.
 *
 * @group AWS Cloud
 */
class VpcPeeringConnectionTest extends AwsCloudTestCase {

  public const AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list aws cloud vpc peering connection',
      'add aws cloud vpc peering connection',
      'view any aws cloud vpc peering connection',
      'edit any aws cloud vpc peering connection',
      'delete any aws cloud vpc peering connection',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getMockDataTemplateVars() {
    return [
      'vpc_peering_connection_id' => 'pcx' . $this->getRandomAwsId(),
    ];
  }

  /**
   * Tests CRUD for VPC Peering Connection information.
   */
  public function testVpcPeeringConnection() {
    $cloud_context = $this->cloudContext;

    // List VPC Peering Connection for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/vpc_peering_connection");
    $this->assertResponse(200);

    $vpc_ids = $this->updateVpcsMockData(self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT);

    // Add a new Vpc Peering Connection.
    $add = $this->createVpcPeeringConnectionTestFormData(self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT, $vpc_ids);
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $vpc_peering_connection_id = $this->latestTemplateVars['vpc_peering_connection_id'];
      $this->addVpcPeeringConnectionMockData($add[$i], $vpc_peering_connection_id);

      $num = $i + 1;

      $this->drupalPostForm(
        "/clouds/aws_cloud/$cloud_context/vpc_peering_connection/add",
        $add[$i],
        $this->t('Save')
      );
      $this->assertResponse(200);
      $this->assertSession()->pageTextContains($this->t('AWS Cloud VPC Peering Connection @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/vpc_peering_connection");
      $this->assertResponse(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);

      $add[$i]['vpc_peering_connection_id'] = $vpc_peering_connection_id;
    }

    // Edit a Vpc Peering Connection.
    $edit = $this->createVpcPeeringConnectionTestFormData(self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT, $vpc_ids);
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      unset($edit[$i]['requester_vpc_id']);
      unset($edit[$i]['accepter_vpc_id']);

      $this->modifyVpcPeeringConnectionMockData($i, $edit[$i]);

      $this->drupalPostForm(
        "/clouds/aws_cloud/$cloud_context/vpc_peering_connection/$num/edit",
        $edit[$i],
        $this->t('Save')
      );
      $this->assertResponse(200);
      $this->assertSession()->pageTextContains($this->t(
        'AWS Cloud VPC Peering Connection @name has been updated.',
        ['@name' => $edit[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/vpc_peering_connection");
      $this->assertResponse(200);
      $this->assertSession()->pageTextContains($edit[$i]['name']);
    }

    // Update tags for empty.
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      // Update tags.
      $this->updateTagsInMockData($num - 1, 'VpcPeeringConnections', 'Name', '', FALSE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/vpc_peering_connection");
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkExists($edit[$i]['name']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['vpc_peering_connection_id']);
    }

    // Delete name tags.
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      // Update tags.
      $this->updateTagsInMockData($num - 1, 'VpcPeeringConnections', 'Name', '', TRUE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/vpc_peering_connection");
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['vpc_peering_connection_id']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['vpc_peering_connection_id']);
    }

    // Update tags.
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      $this->updateTagsInMockData($num - 1, 'VpcPeeringConnections', 'Name', $edit[$i]['name'], FALSE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/vpc_peering_connection");
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['vpc_peering_connection_id']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkExists($edit[$i]['name']);
    }

    // Delete Vpc Peering Connection.
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteVpcPeeringConnectionMockData($i);

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/vpc_peering_connection/$num/delete");
      $this->assertResponse(200);

      $this->drupalPostForm(
        "/clouds/aws_cloud/$cloud_context/vpc_peering_connection/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertResponse(200);
      $this->assertSession()->pageTextContains($this->t(
        'The AWS Cloud VPC Peering Connection @name has been deleted.',
        ['@name' => $edit[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/vpc_peering_connection");
      $this->assertResponse(200);
      $this->assertSession()->pageTextNotContains($edit[$i]['name']);
    }
  }

  /**
   * Tests deleting VPC Peering Connections with bulk operation.
   */
  public function testVpcPeeringConnectionBulk() {
    $cloud_context = $this->cloudContext;
    for ($i = 0; $i < self::AWS_CLOUD_VPC_PEERING_CONNECTION_REPEAT_COUNT; $i++) {
      // Create VPC Peering Connections.
      $vpc_peering_connections = $this->createVpcPeeringConnectionsRandomTestFormData();
      $index = 0;
      $entities = [];
      foreach ($vpc_peering_connections as $vpc_peering_connection) {
        $entities[] = $this->createVpcPeeringConnectionTestEntity(
          $index++,
          $vpc_peering_connection['VpcPeeringConnectionId'],
          $vpc_peering_connection['Name'],
          $cloud_context
        );
      }
      $this->doTestEntityBulk('vpc_peering_connection', $entities);
    }
  }

}
