<?php

namespace Drupal\Tests\aws_cloud\Unit\Service\Ec2;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Session\AccountInterface;
use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Lock\LockBackendInterface;
use Drupal\Tests\UnitTestCase;

/**
 * Tests AWS Cloud Service.
 *
 * @group AWS Cloud
 */
class Ec2ServiceTest extends UnitTestCase {

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    $this->service = new Ec2ServiceMock(
      $this->createMock(EntityTypeManagerInterface::class),
      $this->createMock(LoggerChannelFactoryInterface::class),
      $this->getConfigFactoryStub([
        'aws_cloud.settings' => ['aws_cloud_test_mode' => TRUE],
      ]),
      $this->getMockBuilder(Messenger::class)
        ->disableOriginalConstructor()
        ->getMock(),
      $this->getStringTranslationStub(),
      $this->createMock(AccountInterface::class),
      $this->createMock(CloudConfigPluginManagerInterface::class),
      $this->createMock(FieldTypePluginManagerInterface::class),
      $this->createMock(EntityFieldManagerInterface::class),
      $this->createMock(LockBackendInterface::class)
    );
  }

  /**
   * Testing get availability zones.
   */
  public function testGetAvailabilityZones() {
    $random = $this->getRandomGenerator();

    $zones = [];
    $zones[] = $random->name(8, TRUE);
    $zones[] = $random->name(8, TRUE);
    $zones[] = $random->name(8, TRUE);

    $responseZones = [];
    $responseZones['AvailabilityZones'] = array_map(static function ($zone) {
      return ['ZoneName' => $zone];
    }, $zones);
    $this->service->setAvailabilityZonesForTest($responseZones);

    $expectedResult = array_combine($zones, $zones);
    $actualResult = $this->service->getAvailabilityZones();
    $this->assertSame($expectedResult, $actualResult);
  }

}
