<?php

namespace Drupal\k8s\Plugin\Validation\Constraint;

use Drupal\Component\Serialization\Yaml;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Validates YAML URL fields.
 */
class YamlUrlConstraintValidator extends ConstraintValidator {

  /**
   * {@inheritdoc}
   */
  public function validate($entity, Constraint $constraint) {

    // Only perform validations for K8s bundles.
    if ($entity->bundle() === 'k8s') {
      $yaml_url = $entity->get('field_yaml_url')->getValue();
      $detail = $entity->get('field_detail')->getValue();
      $source_type = $entity->get('field_source_type')->getValue();

      if (!empty($source_type) && $source_type[0]['value'] === 'git') {
        if (empty($yaml_url)) {
          $this->context
            ->buildViolation($constraint->requiredResoueceUrl)
            ->atPath('field_yaml_url')
            ->addViolation();
        }
      }
      else {
        if (empty($yaml_url) && empty($detail)) {
          $this->context
            ->buildViolation($constraint->requiredYamlUrlOrDetail)
            ->atPath('field_yaml_url')
            ->addViolation();
        }
        elseif (!empty($yaml_url) && !empty($detail)) {
          $this->context
            ->buildViolation($constraint->prohibitYamlUrlAndDetail)
            ->atPath('field_yaml_url')
            ->addViolation();
        }
        elseif (!empty($yaml_url)) {
          $uri = $yaml_url[0]['uri'];
          $file = file_get_contents($uri);
          if (!$file) {
            $this->context
              ->buildViolation($constraint->invalidYamlUrl)
              ->atPath('field_yaml_url')
              ->addViolation();
          }
          else {
            try {
              $yaml = Yaml::decode($file);
              if (!is_array($yaml)) {
                $this->context
                  ->buildViolation($constraint->invalidYamlFormat)
                  ->atPath('field_yaml_url')
                  ->addViolation();
              }
              elseif (!isset($yaml['kind'])) {
                $this->context
                  ->buildViolation($constraint->noKindFound)
                  ->atPath('field_yaml_url')
                  ->addViolation();
              }
              else {
                $kind = $yaml['kind'];
                $templates = k8s_supported_cloud_server_templates();
                $object = array_search($kind, $templates);
                if ($object === FALSE) {
                  $this->context
                    ->buildViolation($constraint->unsupportedObjectType)
                    ->atPath('field_yaml_url')
                    ->addViolation();
                }
              }
            }
            catch (\Exception $e) {
              $this->context
                ->buildViolation($constraint->invalidYaml . $e->getMessage())
                ->atPath('field_yaml_url')
                ->addViolation();
            }
          }
        }
      }
    }
  }

}
