<?php

namespace Drupal\Tests\k8s\Functional\Config;

use Drupal\Tests\k8s\Functional\K8sTestCase;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;

/**
 * Tests cloud service provider.
 *
 * @group K8s
 */
class K8sCloudConfigTest extends K8sTestCase {

  public const K8S_CLOUD_CONFIG_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'administer cloud service providers',
      'view all cloud service providers',
      'add cloud service providers',
      'edit cloud service providers',
      'edit own cloud service providers',
      'delete cloud service providers',
      'delete own cloud service providers',
      'view unpublished cloud service providers',
      'view own unpublished cloud service providers',
      'view published cloud service providers',
      'view own published cloud service providers',
      'access dashboard',
      'view cloud service provider admin list',
      'administer k8s',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function createCloudContext($bundle_type = 'k8s') {
  }

  /**
   * Tests CRUD for cloud service provider information.
   */
  public function testCloudConfig() {

    // List k8s.
    $this->drupalGet('/admin/structure/cloud_config');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextNotContains($this->t('Notice'));
    $this->assertSession()->pageTextNotContains($this->t('Warning'));

    // Add a new Config information.
    $add = $this->createCloudConfigTestFormData(self::K8S_CLOUD_CONFIG_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_CLOUD_CONFIG_REPEAT_COUNT; $i++) {

      unset($add[$i]['cloud_context']);
      $this->drupalGet('/admin/structure/cloud_config/add');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));

      // Test if API server field is empty.
      $_add = $add;
      $_add[$i]['field_api_server[0][value]'] = '';
      $this->drupalPostForm('/admin/structure/cloud_config/add',
        $_add[$i],
        $this->t('Save')
      );

      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Error'));
      $this->assertSession()->pageTextContains($this->t('The API server field cannot be empty.'));

      // Test if API server field is empty.
      $_add = $add;
      $_add[$i]['field_token[0][value]'] = '';
      $this->drupalPostForm('/admin/structure/cloud_config/add',
        $_add[$i],
        $this->t('Save')
      );

      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Error'));
      $this->assertSession()->pageTextContains($this->t('The token field cannot be empty.'));

      // Test if both API server field and Token field are empty.
      $_add = $add;
      $_add[$i]['field_api_server[0][value]'] = '';
      $_add[$i]['field_token[0][value]'] = '';
      $this->drupalPostForm('/admin/structure/cloud_config/add',
        $_add[$i],
        $this->t('Save')
      );

      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Error'));
      $this->assertSession()->pageTextContains($this->t('The API server field cannot be empty.'));
      $this->assertSession()->pageTextContains($this->t('The token field cannot be empty.'));

      // Test the normal "Save" case.
      $this->drupalPostForm('/admin/structure/cloud_config/add',
        $add[$i],
        $this->t('Save')
      );

      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->pageTextContains($this->t('Created the cloud service provider: @label.',
        ['@label' => $add[$i]['name[0][value]']]
      ));

      // Make sure listing for '/admin/structure/cloud_config'.
      $this->drupalGet('/admin/structure/cloud_config');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($add[$i]['name[0][value]']);

      // Make sure listing for '/clouds'.
      $this->drupalGet('/clouds');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($add[$i]['name[0][value]']);

      // Make sure listing for '/admin/config/services/cloud/k8s'.
      $this->drupalGet('/admin/config/services/cloud/k8s');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($add[$i]['name[0][value]']);
    }

    // Edit Config case.
    $edit = $this->createCloudConfigTestFormData(self::K8S_CLOUD_CONFIG_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_CLOUD_CONFIG_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      unset($edit[$i]['cloud_context']);

      $this->drupalPostForm("/admin/structure/cloud_config/${num}/edit",
        $edit[$i],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));

      // Make sure listing for '/admin/structure/cloud_config'.
      $this->drupalGet('/admin/structure/cloud_config');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($edit[$i]['name[0][value]']);

      // Make sure listing for '/clouds'.
      $this->drupalGet('/clouds');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($edit[$i]['name[0][value]']);

      // Make sure listing for '/admin/config/services/cloud/k8s'.
      $this->drupalGet('/admin/config/services/cloud/k8s');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($edit[$i]['name[0][value]']);
    }

    // Delete Config Items.
    for ($i = 0; $i < self::K8S_CLOUD_CONFIG_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->drupalPostForm("/admin/structure/cloud_config/${num}/delete",
        [],
        $this->t('Delete')
      );

      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->pageTextContains($this->t('The cloud service provider @label has been deleted.',
        ['@label' => $edit[$i]['name[0][value]']]
      ));

      // Because $cloud_context has been deleted.
      // Make sure listing.
      for ($k = 0; $k < $num; $k++) {

        // Make sure listing for '/admin/structure/cloud_config'.
        $this->drupalGet('/admin/structure/cloud_config');
        $this->assertSession()->statusCodeEquals(200);
        $this->assertSession()->pageTextNotContains($this->t('Notice'));
        $this->assertSession()->pageTextNotContains($this->t('Warning'));
        $this->assertSession()->linkNotExists($edit[$k]['name[0][value]']);

        // Make sure listing for '/clouds'.
        $this->drupalGet('/clouds');
        $this->assertSession()->statusCodeEquals(200);
        $this->assertSession()->pageTextNotContains($this->t('Notice'));
        $this->assertSession()->pageTextNotContains($this->t('Warning'));
        $this->assertSession()->linkNotExists($edit[$k]['name[0][value]']);

        // Make sure listing for '/admin/config/services/cloud/k8s'.
        $this->drupalGet('/admin/config/services/cloud/k8s');
        $this->assertSession()->statusCodeEquals(200);
        $this->assertSession()->pageTextNotContains($this->t('Notice'));
        $this->assertSession()->pageTextNotContains($this->t('Warning'));
        $this->assertSession()->linkNotExists($edit[$k]['name[0][value]']);
      }
    }
  }

  /**
   * Tests deleting cloud service provider (CloudConfig) with bulk operation.
   */
  public function testCloudConfigBulk() {

    for ($i = 0; $i < self::K8S_CLOUD_CONFIG_REPEAT_COUNT; $i++) {
      // Create cloud service providers.
      $cloud_configs = $this->createCloudConfigRandomTestFormData();
      $index = 0;
      $entities = [];
      foreach ($cloud_configs ?: [] as $cloud_config) {
        $entity = $this->createCloudConfigTestEntity($index++, $cloud_config['CloudContext'], $cloud_config['Name']);

        $this->grantPermissions(
          Role::load(
          RoleInterface::AUTHENTICATED_ID),
          ["view {$entity->getCloudContext()}"]
        );
        $entities[] = $entity;
      }

      $this->drupalGet('/admin/config/services/cloud/k8s');
      $this->doTestEntityBulk(
        'cloud_config',
        $entities,
        'delete',
        'deleted',
        '/admin/structure'
      );

      $this->drupalGet('/admin/config/services/cloud/k8s');
      foreach ($cloud_configs ?: [] as $cloud_config) {
        $this->assertSession()->linkNotExists($cloud_config['Name']);
      }

      $this->drupalGet('/clouds');
      foreach ($cloud_configs ?: [] as $cloud_config) {
        $this->assertSession()->linkNotExists($cloud_config['Name']);
      }
    }
  }

}
