<?php

namespace Drupal\Tests\k8s\Functional\Form;

use Drupal\Tests\k8s\Functional\K8sTestCase;

/**
 * Tests K8sProjectCreateForm.
 *
 * @group K8s
 */
class K8sProjectCreateFormTest extends K8sTestCase {

  public const K8S_PROJECT_REPEAT_COUNT = 3;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list k8s namespace',
      'view k8s namespace',
      'list k8s resource quota',
      'view k8s resource quota',
      'create k8s project',
    ];
  }

  /**
   * Tests creating a project.
   */
  public function testCreateProject() {

    $cloud_context = $this->cloudContext;

    // List Namespace for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/namespace");
    $this->assertSession()->statusCodeEquals(200);

    // Create a new project.
    $add = $this->createProjectTestFormData(self::K8S_PROJECT_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_PROJECT_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $namespace = [];
      $namespace['name'] = $add[$i]['project_name'];

      $resource_quota = [];
      $resource_quota['name'] = $add[$i]['project_name'];
      $resource_quota['post_data']['namespace'] = $add[$i]['project_name'];

      if (!empty($add[$i]['k8s_enable_scheduling_time_based'])) {
        $namespace['metadata']['annotations']['startup_time'] = $add[$i]['k8s_startup_time_hour'] . ':' . $add[$i]['k8s_startup_time_minute'];
        $namespace['metadata']['annotations']['stop_time'] = $add[$i]['k8s_stop_time_hour'] . ':' . $add[$i]['k8s_stop_time_minute'];
      }
      if (!empty($add[$i]['k8s_enable_scheduling_resource_based'])) {
        if (!empty($add[$i]['k8s_request_cpu'])) {
          $namespace['metadata']['annotations']['request_cpu'] = $add[$i]['k8s_request_cpu'] . 'm';
          $resource_quota['spec']['hard']['cpu'] = $add[$i]['k8s_request_cpu'] . 'm';
        }
        if (!empty($add[$i]['k8s_request_memory'])) {
          $namespace['metadata']['annotations']['request_memory'] = $add[$i]['k8s_request_memory'] . 'Mi';
          $resource_quota['spec']['hard']['memory'] = $add[$i]['k8s_request_memory'] . 'Mi';
        }
        if (!empty($add[$i]['k8s_pod_count'])) {
          $namespace['metadata']['annotations']['pod_count'] = $add[$i]['k8s_pod_count'];
          $resource_quota['spec']['hard']['pods'] = $add[$i]['k8s_pod_count'];
        }
      }

      $this->addNamespaceMockData($namespace);
      if (!empty($add[$i]['k8s_enable_scheduling_resource_based'])) {
        $this->addResourceQuotaMockData($resource_quota);
      }

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/namespace/create_project",
        $add[$i],
        $this->t('Create')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('The project "@name" has been created.', ['@name' => $add[$i]['project_name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/namespace");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['project_name']);

      if (!empty($add[$i]['k8s_enable_scheduling_resource_based'])) {
        $this->drupalGet("/clouds/k8s/$cloud_context/resource_quota");
        $this->assertSession()->statusCodeEquals(200);
        $this->assertSession()->pageTextContains($resource_quota['post_data']['namespace']);
      }
    }
  }

}
