<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s cluster role binding.
 *
 * @group K8s
 */
class K8sClusterRoleBindingTest extends K8sTestCase {

  public const K8S_CLUSTER_ROLE_BINDING_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list k8s cluster role binding',
      'view k8s cluster role binding',
      'edit k8s cluster role binding',
      'add k8s cluster role binding',
      'delete k8s cluster role binding',
    ];
  }

  /**
   * Tests CRUD for Cluster Role Binding.
   */
  public function testClusterRoleBinding() {

    $cloud_context = $this->cloudContext;

    // List Cluster Role Binding for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/cluster_role_binding");
    $this->assertSession()->statusCodeEquals(200);

    // Add a new Cluster Role Binding.
    $add = $this->createClusterRoleBindingTestFormData(self::K8S_CLUSTER_ROLE_BINDING_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_CLUSTER_ROLE_BINDING_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addClusterRoleBindingMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/cluster_role_binding/add",
        $add[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Kubernetes Cluster Role Binding @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/cluster_role_binding");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Cluster Role Binding.
    $edit = $this->createClusterRoleBindingTestFormData(self::K8S_CLUSTER_ROLE_BINDING_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_CLUSTER_ROLE_BINDING_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateClusterRoleBindingMockData($edit[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/cluster_role_binding/$num/edit",
        $edit[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'Kubernetes Cluster Role Binding @name has been updated.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Cluster Role Binding.
    for ($i = 0; $i < self::K8S_CLUSTER_ROLE_BINDING_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteClusterRoleBindingMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/cluster_role_binding/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Kubernetes Cluster Role Binding @name has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/cluster_role_binding");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($add[$i]['name']);
    }
  }

  /**
   * Tests deleting cluster role bindings with bulk operation.
   */
  public function testClusterRoleBindingBulk() {

    for ($i = 0; $i < self::K8S_CLUSTER_ROLE_BINDING_REPEAT_COUNT; $i++) {
      // Create Cluster Role Bindings.
      $cluster_role_bindings = $this->createClusterRoleBindingsRandomTestFormData();
      $entities = [];
      foreach ($cluster_role_bindings ?: [] as $cluster_role_binding) {
        $entities[] = $this->createClusterRoleBindingTestEntity($cluster_role_binding);
      }
      $this->deleteClusterRoleBindingMockData($cluster_role_bindings[0]);
      $this->doTestEntityBulk('cluster_role_binding', $entities);
    }
  }

}
