<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s cluster role.
 *
 * @group K8s
 */
class K8sClusterRoleTest extends K8sTestCase {

  public const K8S_CLUSTER_ROLE_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list k8s cluster role',
      'view k8s cluster role',
      'edit k8s cluster role',
      'add k8s cluster role',
      'delete k8s cluster role',
    ];
  }

  /**
   * Tests CRUD for Cluster Role.
   */
  public function testClusterRole() {

    $cloud_context = $this->cloudContext;

    // List Cluster Role for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/cluster_role");
    $this->assertSession()->statusCodeEquals(200);

    // Add a new Cluster Role.
    $add = $this->createClusterRoleTestFormData(self::K8S_CLUSTER_ROLE_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_CLUSTER_ROLE_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addClusterRoleMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/cluster_role/add",
        $add[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Kubernetes Cluster Role @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/cluster_role");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Cluster Role.
    $edit = $this->createClusterRoleTestFormData(self::K8S_CLUSTER_ROLE_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_CLUSTER_ROLE_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateClusterRoleMockData($edit[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/cluster_role/$num/edit",
        $edit[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'Kubernetes Cluster Role @name has been updated.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Cluster Role.
    for ($i = 0; $i < self::K8S_CLUSTER_ROLE_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteClusterRoleMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/cluster_role/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Kubernetes Cluster Role @name has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/cluster_role");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($add[$i]['name']);
    }
  }

  /**
   * Tests deleting cluster roles with bulk operation.
   */
  public function testClusterRoleBulk() {

    for ($i = 0; $i < self::K8S_CLUSTER_ROLE_REPEAT_COUNT; $i++) {
      // Create Cluster Roles.
      $cluster_roles = $this->createClusterRolesRandomTestFormData();
      $entities = [];
      foreach ($cluster_roles ?: [] as $cluster_role) {
        $entities[] = $this->createClusterRoleTestEntity($cluster_role);
      }
      $this->deleteClusterRoleMockData($cluster_roles[0]);
      $this->doTestEntityBulk('cluster_role', $entities);
    }
  }

}
