<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s daemon set.
 *
 * @group K8s
 */
class K8sDaemonSetTest extends K8sTestCase {

  public const K8S_DAEMON_SET_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    $namespaces = $this->createNamespacesRandomTestFormData();
    $this->createNamespaceTestEntity($namespaces[0]);
    $this->namespace = $namespaces[0]['name'];
    return [
      'list k8s daemon set',
      'add k8s daemon set',
      'edit k8s daemon set',
      'delete k8s daemon set',
      'view k8s namespace ' . $this->namespace,
    ];
  }

  /**
   * Tests CRUD for Daemon Set.
   */
  public function testDaemonSet() {

    $cloud_context = $this->cloudContext;

    // List Daemon Set for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/daemon_set");
    $this->assertSession()->statusCodeEquals(200);

    // Add a new Daemon Set.
    $add = $this->createDaemonSetTestFormData(self::K8S_DAEMON_SET_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_DAEMON_SET_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addDaemonSetMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/daemon_set/add",
        $add[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Kubernetes Daemon Set @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/daemon_set");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Daemon Set.
    $edit = $this->createDaemonSetTestFormData(self::K8S_DAEMON_SET_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_DAEMON_SET_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateDaemonSetMockData($edit[$i]);

      unset($edit[$i]['post_data']['namespace']);
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/daemon_set/$num/edit",
        $edit[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'Kubernetes Daemon Set @name has been updated.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Daemon Set.
    for ($i = 0; $i < self::K8S_DAEMON_SET_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteDaemonSetMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/daemon_set/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Kubernetes Daemon Set @name has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/daemon_set");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($add[$i]['name']);
    }
  }

  /**
   * Tests deleting daemon set with bulk operation.
   */
  public function testDaemonSetBulk() {

    for ($i = 0; $i < self::K8S_DAEMON_SET_REPEAT_COUNT; $i++) {
      // Create Daemon Set.
      $daemon_sets = $this->createDaemonSetsRandomTestFormData();
      $entities = [];
      foreach ($daemon_sets ?: [] as $daemon_set) {
        $entities[] = $this->createDaemonSetTestEntity($daemon_set);
      }
      $this->deleteDaemonSetMockData($daemon_sets[0]);
      $this->doTestEntityBulk('daemon_set', $entities);
    }
  }

}
