<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s endpoint.
 *
 * @group K8s
 */
class K8sEndpointTest extends K8sTestCase {

  public const K8S_ENDPOINT_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    $namespaces = $this->createNamespacesRandomTestFormData();
    $this->createNamespaceTestEntity($namespaces[0]);
    $this->namespace = $namespaces[0]['name'];
    return [
      'list k8s endpoint',
      'add k8s endpoint',
      'edit k8s endpoint',
      'delete k8s endpoint',
      'view k8s namespace ' . $this->namespace,
    ];
  }

  /**
   * Tests CRUD for Endpoint.
   */
  public function testEndpoint() {

    $cloud_context = $this->cloudContext;

    // List endpoint for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/endpoint");
    $this->assertSession()->statusCodeEquals(200);

    // Add a new endpoint.
    $add = $this->createEndpointTestFormData(self::K8S_ENDPOINT_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_ENDPOINT_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addEndpointMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/endpoint/add",
        $add[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Kubernetes Endpoint @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/endpoint");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Endpoint.
    $edit = $this->createEndpointTestFormData(self::K8S_ENDPOINT_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_ENDPOINT_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateEndpointMockData($edit[$i]);

      unset($edit[$i]['post_data']['namespace']);
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/endpoint/$num/edit",
        $edit[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'Kubernetes Endpoint @name has been updated.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Endpoint.
    for ($i = 0; $i < self::K8S_ENDPOINT_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteEndpointMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/endpoint/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Kubernetes Endpoint @name has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/endpoint");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($add[$i]['name']);
    }
  }

  /**
   * Tests deleting endpoint with bulk operation.
   */
  public function testEndpointBulk() {

    for ($i = 0; $i < self::K8S_ENDPOINT_REPEAT_COUNT; $i++) {
      // Create Endpoint.
      $endpoints = $this->createEndpointsRandomTestFormData();
      $entities = [];
      foreach ($endpoints ?: [] as $endpoint) {
        $entities[] = $this->createEndpointTestEntity($endpoint);
      }
      $this->deleteEndpointMockData($endpoints[0]);
      $this->doTestEntityBulk('endpoint', $entities);
    }
  }

}
