<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s ingress.
 *
 * @group K8s
 */
class K8sIngressTest extends K8sTestCase {

  public const K8S_INGRESS_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    $namespaces = $this->createNamespacesRandomTestFormData();
    $this->createNamespaceTestEntity($namespaces[0]);
    $this->namespace = $namespaces[0]['name'];
    return [
      'list k8s ingress',
      'add k8s ingress',
      'edit k8s ingress',
      'delete k8s ingress',
      'view k8s namespace ' . $this->namespace,
    ];
  }

  /**
   * Tests CRUD for Ingress.
   */
  public function testIngress() {

    $cloud_context = $this->cloudContext;

    // List Ingress for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/ingress");
    $this->assertSession()->statusCodeEquals(200);

    // Add a new Ingress.
    $add = $this->createIngressTestFormData(self::K8S_INGRESS_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_INGRESS_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addIngressMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/ingress/add",
        $add[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Kubernetes Ingress @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/ingress");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Ingress.
    $edit = $this->createIngressTestFormData(self::K8S_INGRESS_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_INGRESS_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateIngressMockData($edit[$i]);

      unset($edit[$i]['post_data']['namespace']);
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/ingress/$num/edit",
        $edit[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'Kubernetes Ingress @name has been updated.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Ingress.
    for ($i = 0; $i < self::K8S_INGRESS_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteIngressMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/ingress/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Kubernetes Ingress @name has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/ingress");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($add[$i]['name']);
    }
  }

  /**
   * Tests deleting ingress with bulk operation.
   */
  public function testIngressBulk() {

    for ($i = 0; $i < self::K8S_INGRESS_REPEAT_COUNT; $i++) {
      // Create Ingress.
      $ingresses = $this->createIngressesRandomTestFormData();
      $entities = [];
      foreach ($ingresses ?: [] as $ingress) {
        $entities[] = $this->createIngressTestEntity($ingress);
      }
      $this->deleteIngressMockData($ingresses[0]);
      $this->doTestEntityBulk('ingress', $entities);
    }
  }

}
