<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s Network Policy.
 *
 * @group K8s
 */
class K8sNetworkPolicyTest extends K8sTestCase {

  public const K8S_NETWORK_POLICY_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    $namespaces = $this->createNamespacesRandomTestFormData();
    $this->createNamespaceTestEntity($namespaces[0]);
    $this->namespace = $namespaces[0]['name'];

    return [
      'list k8s network policy',
      'view k8s network policy',
      'edit k8s network policy',
      'add k8s network policy',
      'delete k8s network policy',
      'view k8s namespace ' . $this->namespace,
    ];
  }

  /**
   * Tests CRUD for Network Policy.
   */
  public function testNetworkPolicy() {

    $cloud_context = $this->cloudContext;

    // List Network Policy for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/network_policy");
    $this->assertSession()->statusCodeEquals(200);

    // Add a new Network Policy.
    $add = $this->createNetworkPolicyTestFormData(self::K8S_NETWORK_POLICY_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_NETWORK_POLICY_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addNetworkPolicyMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/network_policy/add",
        $add[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Kubernetes Network Policy @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/network_policy");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Network Policy.
    $edit = $this->createNetworkPolicyTestFormData(self::K8S_NETWORK_POLICY_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_NETWORK_POLICY_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateNetworkPolicyMockData($edit[$i]);

      unset($edit[$i]['post_data']['namespace']);
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/network_policy/$num/edit",
        $edit[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'Kubernetes Network Policy @name has been updated.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Network Policy.
    for ($i = 0; $i < self::K8S_NETWORK_POLICY_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteNetworkPolicyMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/network_policy/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Kubernetes Network Policy @name has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/network_policy");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($add[$i]['name']);
    }
  }

  /**
   * Tests deleting Network Policies with bulk operation.
   */
  public function testNetworkPolicyBulk() {

    for ($i = 0; $i < self::K8S_NETWORK_POLICY_REPEAT_COUNT; $i++) {
      // Create Network Policies.
      $network_policies = $this->createNetworkPoliciesRandomTestFormData($this->namespace);
      $entities = [];
      foreach ($network_policies ?: [] as $network_policy) {
        $entities[] = $this->createNetworkPolicyTestEntity($network_policy);
      }
      $this->deleteNetworkPolicyMockData($network_policies[0]);
      $this->doTestEntityBulk('network_policy', $entities);
    }
  }

}
