<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s secret.
 *
 * @group K8s
 */
class K8sSecretTest extends K8sTestCase {

  public const K8S_SECRET_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    $namespaces = $this->createNamespacesRandomTestFormData();
    $this->createNamespaceTestEntity($namespaces[0]);
    $this->namespace = $namespaces[0]['name'];

    return [
      'list k8s secret',
      'view k8s secret',
      'edit k8s secret',
      'add k8s secret',
      'delete k8s secret',
      'view k8s namespace ' . $this->namespace,
    ];
  }

  /**
   * Tests CRUD for Secret.
   */
  public function testSecret() {

    $cloud_context = $this->cloudContext;

    // List Secret for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/secret");
    $this->assertSession()->statusCodeEquals(200);

    // Add a new Secret.
    $add = $this->createSecretTestFormData(self::K8S_SECRET_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_SECRET_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addSecretMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/secret/add",
        $add[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Kubernetes Secret @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/secret");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Secret.
    $edit = $this->createSecretTestFormData(self::K8S_SECRET_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_SECRET_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateSecretMockData($edit[$i]);

      unset($edit[$i]['post_data']['namespace']);
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/secret/$num/edit",
        $edit[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'Kubernetes Secret @name has been updated.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Secret.
    for ($i = 0; $i < self::K8S_SECRET_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteSecretMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/secret/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Kubernetes Secret @name has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/secret");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($add[$i]['name']);
    }
  }

  /**
   * Tests deleting secrets with bulk operation.
   */
  public function testSecretBulk() {

    for ($i = 0; $i < self::K8S_SECRET_REPEAT_COUNT; $i++) {
      // Create Secrets.
      $secrets = $this->createSecretsRandomTestFormData($this->namespace);
      $entities = [];
      foreach ($secrets ?: [] as $secret) {
        $entities[] = $this->createSecretTestEntity($secret);
      }
      $this->deleteSecretMockData($secrets[0]);
      $this->doTestEntityBulk('secret', $entities);
    }
  }

}
