<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s service.
 *
 * @group K8s
 */
class K8sServiceEntityTest extends K8sTestCase {

  public const K8S_SERVICE_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    $namespaces = $this->createNamespacesRandomTestFormData();
    $this->createNamespaceTestEntity($namespaces[0]);
    $this->namespace = $namespaces[0]['name'];

    return [
      'list k8s service',
      'view k8s service',
      'edit k8s service',
      'add k8s service',
      'delete k8s service',
      'view k8s namespace ' . $this->namespace,
    ];
  }

  /**
   * Tests CRUD for Service.
   */
  public function testService() {

    $cloud_context = $this->cloudContext;

    // List Service for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/service");
    $this->assertSession()->statusCodeEquals(200);

    // Add a new Service.
    $add = $this->createServiceTestFormData(self::K8S_SERVICE_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_SERVICE_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addServiceMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/service/add",
        $add[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Kubernetes Service @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/service");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Service.
    $edit = $this->createServiceTestFormData(self::K8S_SERVICE_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_SERVICE_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateServiceMockData($edit[$i]);

      unset($edit[$i]['post_data']['namespace']);
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/service/$num/edit",
        $edit[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'Kubernetes Service @name has been updated.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Service.
    for ($i = 0; $i < self::K8S_SERVICE_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteServiceMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/service/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Kubernetes Service @name has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/service");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($add[$i]['name']);
    }
  }

  /**
   * Tests deleting services with bulk operation.
   */
  public function testServiceBulk() {

    for ($i = 0; $i < self::K8S_SERVICE_REPEAT_COUNT; $i++) {
      // Create Services.
      $services = $this->createServicesRandomTestFormData($this->namespace);
      $entities = [];
      foreach ($services ?: [] as $service) {
        $entities[] = $this->createServiceTestEntity($service);
      }
      $this->deleteServiceMockData($services[0]);
      $this->doTestEntityBulk('service', $entities);
    }
  }

}
