<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s storage class.
 *
 * @group K8s
 */
class K8sStorageClassTest extends K8sTestCase {

  public const K8S_STORAGE_CLASS_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list k8s storage class',
      'view k8s storage class',
      'edit k8s storage class',
      'add k8s storage class',
      'delete k8s storage class',
    ];
  }

  /**
   * Tests CRUD for Storage Class.
   */
  public function testStorageClass() {

    $cloud_context = $this->cloudContext;

    // List Storage Class for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/storage_class");
    $this->assertSession()->statusCodeEquals(200);

    // Add a new Storage Class.
    $add = $this->createStorageClassTestFormData(self::K8S_STORAGE_CLASS_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_STORAGE_CLASS_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addStorageClassMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/storage_class/add",
        $add[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Kubernetes Storage Class @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/storage_class");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Storage Class.
    $edit = $this->createStorageClassTestFormData(self::K8S_STORAGE_CLASS_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_STORAGE_CLASS_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateStorageClassMockData($edit[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/storage_class/$num/edit",
        $edit[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'Kubernetes Storage Class @name has been updated.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Storage Class.
    for ($i = 0; $i < self::K8S_STORAGE_CLASS_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteStorageClassMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/storage_class/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Kubernetes Storage Class @name has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/storage_class");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($add[$i]['name']);
    }
  }

  /**
   * Tests deleting storage classs with bulk operation.
   */
  public function testStorageClassBulk() {

    for ($i = 0; $i < self::K8S_STORAGE_CLASS_REPEAT_COUNT; $i++) {
      // Create Storage Classs.
      $storage_classes = $this->createStorageClassesRandomTestFormData();
      $entities = [];
      foreach ($storage_classes ?: [] as $storage_class) {
        $entities[] = $this->createStorageClassTestEntity($storage_class);
      }
      $this->deleteStorageClassMockData($storage_classes[0]);
      $this->doTestEntityBulk('storage_class', $entities);
    }
  }

}
