<?php

namespace Drupal\openstack;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Access controller for the OpenStack Cloud Key Pair entity.
 *
 * @see \Drupal\openstack\Entity\KeyPair.
 */
class KeyPairAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    /** @var \Drupal\openstack\Entity\KeyPairInterface $entity */
    switch ($operation) {
      case 'view':
        return AccessResult::allowedIfHasPermission($account, 'view published openstack cloud key pairs');

      case 'edit':
      case 'update':
        return AccessResult::allowedIfHasPermission($account, 'edit openstack cloud key pairs');

      case 'delete':
        return AccessResult::allowedIfHasPermission($account, 'delete openstack cloud key pairs');
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    return AccessResult::allowedIfHasPermission($account, 'add openstack cloud key pairs');
  }

}
