<?php

namespace Drupal\openstack;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Access controller for the OpenStack Cloud NetworkInterface entity.
 *
 * @see \Drupal\openstack\Entity\NetworkInterface.
 */
class NetworkInterfaceAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    /** @var \Drupal\openstack\Entity\NetworkInterfaceInterface $entity */
    switch ($operation) {
      case 'view':
        return AccessResult::allowedIfHasPermission($account, 'view published openstack cloud network interfaces');

      case 'edit':
      case 'update':
        return AccessResult::allowedIfHasPermission($account, 'edit openstack cloud network interfaces');

      case 'delete':
        return AccessResult::allowedIfHasPermission($account, 'delete openstack cloud network interfaces');
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    return AccessResult::allowedIfHasPermission($account, 'add openstack cloud network interfaces');
  }

}
